"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const polyfill_1 = require("../polyfill");
class Handler {
    constructor(input) {
        if (!Array.isArray(input) && !(input instanceof Handler)) {
            throw new TypeError('Invalid input');
        }
        if (input instanceof Handler) {
            this.input = input.value;
        }
        else {
            this.input = input;
        }
    }
    get value() {
        return this.input;
    }
    /**
     * ForEach
     * @param  {any} cb
     * @returns Promise
     */
    forEach(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.callPf(polyfill_1.forEach, cb);
        });
    }
    /**
     * Map
     * @param  {any} cb
     * @returns Promise<any[]>
     */
    map(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Handler(yield this.callPf(polyfill_1.map, cb));
        });
    }
    /**
     * filter
     * @param  {any} cb
     * @returns Promise<any[]>
     */
    filter(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Handler(yield this.callPf(polyfill_1.filter, cb));
        });
    }
    /**
     * find
     * @param  {any} cb
     * @returns Promise<any>
     */
    find(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.callPf(polyfill_1.find, cb);
        });
    }
    /**
     * find
     * @param  {any} cb
     * @returns Promise<any>
     */
    reduce(initialValue, cb) {
        return __awaiter(this, void 0, void 0, function* () {
            if (initialValue === undefined) {
                throw new TypeError('initial value is not defined');
            }
            return yield this.callPf(polyfill_1.reduce, cb, initialValue);
        });
    }
    toJSON() {
        return this.input;
    }
    toString() {
        return this.input.toString();
    }
    /**
     * Check whether the give object is function
     * Source from https://github.com/then/is-promise/blob/master/index.js
     * @param  {any} cb
     * @returns boolean
     */
    isPromise(cb) {
        return (!!cb &&
            (((typeof cb === 'object' || typeof cb === 'function') &&
                typeof cb.then === 'function') ||
                (cb.constructor && cb.constructor.name === 'AsyncFunction')));
    }
    /**
     * Call polly fills
     * @param  {pfFunctions} pfFunction
     * @param  {any} cb
     * @returns Promise
     */
    callPf(pfFunction, cb, initialValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isPromise(cb)) {
                throw new TypeError('Callback is not an async method');
            }
            if (initialValue !== undefined) {
                return pfFunction.apply(this, [this.input, this.input, initialValue, cb]);
            }
            return pfFunction.apply(this, [this.input, this.input, cb]);
        });
    }
}
exports.Handler = Handler;
//# sourceMappingURL=handler.js.map